/*
 * Copyright (c) 2008 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _RMPLIB_H
#define _RMPLIB_H

struct RMP_DEV_STR;

/*
 * RMP state available through CAN messages
 */
typedef struct RMP_DATA_STR {
	int16_t pitch_angle;
	int16_t pitch_rate;
	int16_t roll_angle;
	int16_t roll_rate;
	int16_t lw_velocity;
	int16_t rw_velocity;
	int16_t yaw_rate;
	int16_t servo_frames;
	int32_t integrated_left_wheel;
	int32_t integrated_right_wheel;
	int32_t integrated_fore_aft;
	int32_t integrated_yaw;
	int16_t left_torque;
	int16_t right_torque;
	int16_t operational_mode;
	int16_t controller_gain_schedule;
	int16_t ui_voltage;
	int16_t powerbase_voltage;
	int16_t velocity_command;
	int16_t turn_command;
} RMP_DATA_STR;

/* Error codes */
#define M_rmpLib			(360 << 16)

#define S_rmpLib_NO_MEMORY		(M_rmpLib | 1)
#define S_rmpLib_FTDI_SYSTEM_ERROR	(M_rmpLib | 2)
#define S_rmpLib_BAD_MESSAGE		(M_rmpLib | 3)
#define S_rmpLib_USB_READ_ERROR		(M_rmpLib | 4)
#define S_rmpLib_USB_WRITE_ERROR	(M_rmpLib | 5)
#define S_rmpLib_BAD_MAGIC		(M_rmpLib | 6)

/* Function prototypes */
extern struct RMP_DEV_STR *rmpOpen(char *);
extern int rmpOpenAll(struct RMP_DEV_STR ***);
extern void rmpClose(struct RMP_DEV_STR *);
extern STATUS rmpReadPacket(struct RMP_DEV_STR *, unsigned char *, int block);
extern STATUS rmpReadPacket2(struct RMP_DEV_STR *, unsigned char *);
extern int rmpReadPackets(struct RMP_DEV_STR *);
extern STATUS rmpSendMessage(struct RMP_DEV_STR *, unsigned char *);
extern STATUS rmpSendSpeed(struct RMP_DEV_STR *, short, short);
extern STATUS rmpPowerOff(struct RMP_DEV_STR *);
extern STATUS rmpPowerOn(struct RMP_DEV_STR *);
extern STATUS rmpSendCanMsg(struct RMP_DEV_STR *, short *);
extern void rmpShowRawData(struct RMP_DEV_STR *);
extern RMP_DATA_STR *rmpGetData(struct RMP_DEV_STR *);
extern pthread_mutex_t *rmpGetMutex(struct RMP_DEV_STR *);
#endif
