/*
 * Copyright (c) 2008 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _RMPREG_H
#define _RMPREG_H

#define USB_VENDOR_SEGWAY	0x0403
#define USB_PRODUCT_RMP		0xe729

/* Magic number to check for device validity */
#define RMP_MAGIC 0x524d5000

/*
 * RMP Control messages 
 */

#define RMP_HEARTBEAT 0x0688
#define RMP_SHUTDOWN 0x0412
#define RMP_CONTROL 0x0413
#define   RMP_COMMAND_NONE 0
#define	  RMP_SET_MAX_VELOCITY_SCALE_FACTOR 10
#define   RMP_SET_MAX_ACCELERATION_SCALE_FACTOR 11
#define   RMP_SET_MAC_TURN_RATE_SCALE_FACTOR 12
#define   RMP_SET_GAIN_SCHEDULE 13
#define   RMP_SET_CURRENT_LIMIT_SCALE_FACTOR 14
#define   RMP_SET_BALANCE_MODE_LOCKOUT 15
#define     RMP_BALANCE_ALLOWED 0
#define     RMP_LOCKOUT_BALANCE 1
#define   RMP_SET_OPERATIONAL_MODE 16
#define     RMP_TRACTOR_MODE 1
#define     RMP_BALANCE_MODE 2
#define     RMP_POWER_DOWN 3
#define   RMP_RESET_INTEGRATORS 50
#define     RMP_RIGHT_WHEEL 1
#define     RMP_LEFT_WHEEL 2
#define     RMP_YAW 4
#define     RMP_FORE_AFT 8
#define RMP_CU_STATUS 0x0680
#define RMP_MONITOR_DATA_1 0x0400
#define RMP_MONITOR_DATA_2 0x0401
#define RMP_MONITOR_DATA_3 0x0402
#define RMP_MONITOR_DATA_4 0x0403
#define RMP_MONITOR_DATA_5 0x0404
#define RMP_MONITOR_DATA_6 0x0405
#define RMP_MONITOR_DATA_7 0x0406
#define RMP_MONITOR_DATA_8 0x0407

/*
 * USB Message constants
 */
/* Read chunk size */
#define RMP_USB_CHUNK_SIZE 4096

/* Paket size */
#define RMP_USB_PACKET_SIZE 18

/* Byte 0 */
#define RMP_USB_MESSAGE_HEADER 0xf0
/* Byte 1 */
#define RMP_USB_COMMAND 0x55
/* Byte 2 */
#define RMP_USB_CANA_DEV 0x00
#define RMP_USB_CMD_RESET 0x05

struct usb_message {
	SIMPLEQ_ENTRY(usb_message) next;
        unsigned char data[RMP_USB_PACKET_SIZE];
};

/*
 * Descriptor of a RMP connection
 */
struct RMP_DEV_STR {
	unsigned int magic;
	struct ftdi_context *ftdi_context;
	SIMPLEQ_HEAD(paquetQueue, usb_message) paquetQueue;
	unsigned char readBuf[RMP_USB_CHUNK_SIZE];
        int readPtr;
	pthread_mutex_t mutex;
	struct RMP_DATA_STR *state;
	int npackets;
	char *serial;
};

#endif
